@@ -1,4 +1,4 @@
-# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
 # Copyright 2002 Ben Escoto <ben@emerose.org>
 # Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
@@ -19,12 +19,11 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-u"""Parse command line, check for consistency, and set globals"""
+u"""Parse command line, check for consistency, and set config"""
 
 from __future__ import print_function
 from future import standard_library
 standard_library.install_aliases()
-from builtins import filter
 from builtins import str
 from builtins import range
 
@@ -43,7 +42,7 @@ except ImportError:
 
 from duplicity import backend
 from duplicity import dup_time
-from duplicity import globals
+from duplicity import config
 from duplicity import gpg
 from duplicity import log
 from duplicity import path
@@ -76,28 +75,32 @@ commands = [u"cleanup",
 
 
 def old_fn_deprecation(opt):
-    log.Log(_(u"Warning: Option %s is pending deprecation "
-              u"and will be removed in a future release.\n"
+    log.Log(_(u"Warning: Option %s is pending deprecation and will be removed in version 2.0.\n"
               u"Use of default filenames is strongly suggested.") % opt,
-            log.ERROR, force_print=True)
+            log.WARNING, force_print=True)
 
 
 def old_globbing_filelist_deprecation(opt):
-    log.Log(_(u"Warning: Option %s is pending deprecation and will be removed in a future release.\n"
+    log.Log(_(u"Warning: Option %s is pending deprecation and will be removed in version 2.0.\n"
               u"--include-filelist and --exclude-filelist now accept globbing characters and should "
               u"be used instead.") % opt,
-            log.ERROR, force_print=True)
+            log.WARNING, force_print=True)
 
 
 def stdin_deprecation(opt):
     # See https://bugs.launchpad.net/duplicity/+bug/1423367
     # In almost all Linux distros stdin is a file represented by /dev/stdin,
     # so --exclude-file=/dev/stdin will work as a substitute.
-    log.Log(_(u"Warning: Option %s is pending deprecation and will be removed in a future release.\n"
+    log.Log(_(u"Warning: Option %s is pending deprecation and will be removed in version 2.0.\n"
               u"On many GNU/Linux systems, stdin is represented by /dev/stdin and\n"
               u"--include-filelist=/dev/stdin or --exclude-filelist=/dev/stdin could\n"
               u"be used as a substitute.") % opt,
-            log.ERROR, force_print=True)
+            log.WARNING, force_print=True)
+
+
+# log options handled in log.py.  Add noop to make optparse happy
+def noop():
+    pass
 
 
 def expand_fn(filename):
@@ -108,8 +111,8 @@ def expand_archive_dir(archdir, backname):
     u"""
     Return expanded version of archdir joined with backname.
     """
-    assert globals.backup_name is not None, \
-        u"expand_archive_dir() called prior to globals.backup_name being set"
+    assert config.backup_name is not None, \
+        u"expand_archive_dir() called prior to config.backup_name being set"
 
     return expand_fn(os.path.join(archdir, backname))
 
@@ -135,18 +138,18 @@ def generate_default_backup_name(backend_url):
     return burlhash.hexdigest()
 
 
-def check_file(option, opt, value):
+def check_file(option, opt, value):  # pylint: disable=unused-argument
     return expand_fn(value)
 
 
-def check_time(option, opt, value):
+def check_time(option, opt, value):  # pylint: disable=unused-argument
     try:
         return dup_time.genstrtotime(value)
     except dup_time.TimeException as e:
         raise optparse.OptionValueError(str(e))
 
 
-def check_verbosity(option, opt, value):
+def check_verbosity(option, opt, value):  # pylint: disable=unused-argument
     fail = False
 
     value = value.lower()
@@ -218,17 +221,17 @@ def parse_cmdline_options(arglist):
     def set_time_sep(sep, opt):
         if sep == u'-':
             raise optparse.OptionValueError(u"Dash ('-') not valid for time-separator.")
-        globals.time_separator = sep
+        config.time_separator = sep
         old_fn_deprecation(opt)
 
-    def add_selection(o, option, additional_arg, p):
+    def add_selection(o, option, additional_arg, p):  # pylint: disable=unused-argument
         if o.type in (u"string", u"file"):
             addarg = util.fsdecode(additional_arg)
         else:
             addarg = additional_arg
         select_opts.append((util.fsdecode(option), addarg))
 
-    def add_filelist(o, s, filename, p):
+    def add_filelist(o, s, filename, p):  # pylint: disable=unused-argument
         select_opts.append((util.fsdecode(s), util.fsdecode(filename)))
         try:
             select_files.append(io.open(filename, u"rt", encoding=u"UTF-8"))
@@ -236,12 +239,13 @@ def parse_cmdline_options(arglist):
             log.FatalError(_(u"Error opening file %s") % filename,
                            log.ErrorCode.cant_open_filelist)
 
-    def print_ver(o, s, v, p):
-        print(u"duplicity %s" % (globals.version))
+    def print_ver(o, s, v, p):  # pylint: disable=unused-argument
+        print(u"duplicity %s" % config.version)
         sys.exit(0)
 
-    def add_rename(o, s, v, p):
-        globals.rename[os.path.normcase(os.path.normpath(v[0]))] = v[1]
+    def add_rename(o, s, v, p):  # pylint: disable=unused-argument
+        key = util.fsencode(os.path.normcase(os.path.normpath(v[0])))
+        config.rename[key] = util.fsencode(v[1])
 
     parser = optparse.OptionParser(option_class=DupOption, usage=usage())
 
@@ -282,20 +286,20 @@ def parse_cmdline_options(arglist):
     # --encrypt-key <gpg_key_id>
     parser.add_option(u"--encrypt-key", type=u"string", metavar=_(u"gpg-key-id"),
                       dest=u"", action=u"callback",
-                      callback=lambda o, s, v, p: globals.gpg_profile.recipients.append(v))  # @UndefinedVariable
+                      callback=lambda o, s, v, p: config.gpg_profile.recipients.append(v))
 
     # secret keyring in which the private encrypt key can be found
     parser.add_option(u"--encrypt-secret-keyring", type=u"string", metavar=_(u"path"))
 
     parser.add_option(u"--encrypt-sign-key", type=u"string", metavar=_(u"gpg-key-id"),
                       dest=u"", action=u"callback",
-                      callback=lambda o, s, v, p: (globals.gpg_profile.recipients.append(v), set_sign_key(v)))
+                      callback=lambda o, s, v, p: (config.gpg_profile.recipients.append(v), set_sign_key(v)))
 
-    # TRANSL: Used in usage help to represent a "glob" style pattern for
+    # TRANSL: Used in usage help to represent a pattern for
     # matching one or more files, as described in the documentation.
     # Example:
-    # --exclude <shell_pattern>
-    parser.add_option(u"--exclude", action=u"callback", metavar=_(u"shell_pattern"),
+    # --exclude <pattern>
+    parser.add_option(u"--exclude", action=u"callback", metavar=_(u"pattern"),
                       dest=u"", type=u"string", callback=add_selection)
 
     parser.add_option(u"--exclude-device-files", action=u"callback",
@@ -335,7 +339,7 @@ def parse_cmdline_options(arglist):
     parser.add_option(u"--fail-on-volume", type=u"int",
                       help=optparse.SUPPRESS_HELP)
 
-    # used to provide a prefix on top of the defaul tar file name
+    # used to provide a prefix on top of the default tar file name
     parser.add_option(u"--file-prefix", type=u"string", dest=u"file_prefix", action=u"store")
 
     # used to provide a suffix for manifest files only
@@ -347,6 +351,31 @@ def parse_cmdline_options(arglist):
     # used to provide a suffix for sigature files only
     parser.add_option(u"--file-prefix-signature", type=u"string", dest=u"file_prefix_signature", action=u"store")
 
+    # File selection mode switch, changes the interpretation of any subsequent
+    # --exclude* or --include* options to shell globbing.
+    parser.add_option(u"--filter-globbing", action=u"callback",
+                      callback=lambda o, s, v, p: select_opts.append((s, None)))
+
+    # File selection mode switch, changes the interpretation of any subsequent
+    # --exclude* or --include* options to case-insensitive matching.
+    parser.add_option(u"--filter-ignorecase", action=u"callback",
+                      callback=lambda o, s, v, p: select_opts.append((s, None)))
+
+    # File selection mode switch, changes the interpretation of any subsequent
+    # --exclude* or --include* options to literal strings.
+    parser.add_option(u"--filter-literal", action=u"callback",
+                      callback=lambda o, s, v, p: select_opts.append((s, None)))
+
+    # File selection mode switch, changes the interpretation of any subsequent
+    # --exclude* or --include* options to regular expressions.
+    parser.add_option(u"--filter-regexp", action=u"callback",
+                      callback=lambda o, s, v, p: select_opts.append((s, None)))
+
+    # File selection mode switch, changes the interpretation of any subsequent
+    # --exclude* or --include* options to case-sensitive matching.
+    parser.add_option(u"--filter-strictcase", action=u"callback",
+                      callback=lambda o, s, v, p: select_opts.append((s, None)))
+
     # used in testing only - skips upload for a given volume
     parser.add_option(u"--skip-volume", type=u"int",
                       help=optparse.SUPPRESS_HELP)
@@ -359,6 +388,10 @@ def parse_cmdline_options(arglist):
                       metavar=_(u"path"), dest=u"restore_dir",
                       callback=lambda o, s, v, p: setattr(p.values, u"restore_dir", util.fsencode(v.strip(u'/'))))
 
+    # Defines the backup source as a sub-set of the source folder
+    parser.add_option(u"--files-from", type=u"file", metavar=_(u"filename"),
+                      dest=u"", action=u"callback", callback=add_filelist)
+
     # Used to confirm certain destructive operations like deleting old files.
     parser.add_option(u"--force", action=u"store_true")
 
@@ -382,7 +415,10 @@ def parse_cmdline_options(arglist):
     # --hidden-encrypt-key <gpg_key_id>
     parser.add_option(u"--hidden-encrypt-key", type=u"string", metavar=_(u"gpg-key-id"),
                       dest=u"", action=u"callback",
-                      callback=lambda o, s, v, p: globals.gpg_profile.hidden_recipients.append(v))  # @UndefinedVariable
+                      callback=lambda o, s, v, p: config.gpg_profile.hidden_recipients.append(v))
+
+    # Fake-root for iDrived backend
+    parser.add_option(u"--idr-fakeroot", dest=u"fakeroot", type=u"file", metavar=_(u"path"))
 
     # ignore (some) errors during operations; supposed to make it more
     # likely that you are able to restore data under problematic
@@ -405,7 +441,7 @@ def parse_cmdline_options(arglist):
     # TRANSL: Used in usage help to represent an imap mailbox
     parser.add_option(u"--imap-mailbox", metavar=_(u"imap_mailbox"))
 
-    parser.add_option(u"--include", action=u"callback", metavar=_(u"shell_pattern"),
+    parser.add_option(u"--include", action=u"callback", metavar=_(u"pattern"),
                       dest=u"", type=u"string", callback=add_selection)
     parser.add_option(u"--include-filelist", type=u"file", metavar=_(u"filename"),
                       dest=u"", action=u"callback", callback=add_filelist)
@@ -431,6 +467,10 @@ def parse_cmdline_options(arglist):
                       dest=u"", action=u"callback",
                       callback=lambda o, s, v, p: log.add_file(v))
 
+    # log option to add timestamp and level to log entries
+    parser.add_option(u"--log-timestamp", action=u"callback",
+                      callback=lambda o, s, v, p: noop())
+
     # Maximum block size for large files
     parser.add_option(u"--max-blocksize", type=u"int", metavar=_(u"number"))
 
@@ -459,6 +499,10 @@ def parse_cmdline_options(arglist):
     # File owner uid keeps number from tar file. Like same option in GNU tar.
     parser.add_option(u"--numeric-owner", action=u"store_true")
 
+    # Do no restore the uid/gid when finished, useful if you're restoring
+    # data without having root privileges or Unix users support
+    parser.add_option(u"--do-not-restore-ownership", action=u"store_true")
+
     # Whether the old filename format is in effect.
     parser.add_option(u"--old-filenames", action=u"callback",
                       dest=u"old_filenames",
@@ -476,6 +520,9 @@ def parse_cmdline_options(arglist):
     # Verbatim par2 options
     parser.add_option(u"--par2-options", action=u"extend", metavar=_(u"options"))
 
+    # Number of par2 volumes
+    parser.add_option(u"--par2-volumes", type=u"int", metavar=_(u"number"))
+
     # Used to display the progress for the full and incremental backup operations
     parser.add_option(u"--progress", action=u"store_true")
 
@@ -512,6 +559,9 @@ def parse_cmdline_options(arglist):
     # Whether to use S3 Glacier Storage
     parser.add_option(u"--s3-use-glacier", action=u"store_true")
 
+    # Whether to use S3 Glacier IR Storage
+    parser.add_option(u"--s3-use-glacier-ir", action=u"store_true")
+
     # Whether to use S3 Glacier Deep Archive Storage
     parser.add_option(u"--s3-use-deep-archive", action=u"store_true")
 
@@ -552,6 +602,10 @@ def parse_cmdline_options(arglist):
     parser.add_option(u"--s3-kms-key-id", action=u"store", dest=u"s3_kms_key_id")
     parser.add_option(u"--s3-kms-grant", action=u"store", dest=u"s3_kms_grant")
 
+    # Options for specifying region and endpoint of s3
+    parser.add_option(u"--s3-region-name", type=u"string", dest=u"s3_region_name", action=u"store")
+    parser.add_option(u"--s3-endpoint-url", type=u"string", dest=u"s3_endpoint_url", action=u"store")
+
     # Option to specify a Swift container storage policy.
     parser.add_option(u"--swift-storage-policy", type=u"string", metavar=_(u"policy"))
 
@@ -582,6 +636,9 @@ def parse_cmdline_options(arglist):
     # allow the user to switch cloudfiles backend
     parser.add_option(u"--cf-backend", metavar=_(u"pyrax|cloudfiles"))
 
+    # Option that causes the B2 backend to hide files instead of deleting them
+    parser.add_option(u"--b2-hide-files", action=u"store_true")
+
     # If set, use short (< 30 char) filenames for all the remote files.
     parser.add_option(u"--short-filenames", action=u"callback",
                       dest=u"short_filenames",
@@ -605,6 +662,9 @@ def parse_cmdline_options(arglist):
     parser.add_option(u"--ssl-cacert-path", metavar=_(u"path to a folder with certificate authority files"))
     parser.add_option(u"--ssl-no-check-certificate", action=u"store_true")
 
+    # header options for Webdav
+    parser.add_option(u"--webdav-headers", metavar=_(u"extra headers for Webdav, like 'Cookie,name=value'"))
+
     # Working directory for the tempfile module. Defaults to /tmp on most systems.
     parser.add_option(u"--tempdir", dest=u"temproot", type=u"file", metavar=_(u"path"))
 
@@ -630,50 +690,81 @@ def parse_cmdline_options(arglist):
 
     parser.add_option(u"-V", u"--version", action=u"callback", callback=print_ver)
 
+    # option for mediafire to purge files on delete instead of sending to trash
+    parser.add_option(u"--mf-purge", action=u"store_true")
+
+    def set_mpsize(o, s, v, p):  # pylint: disable=unused-argument
+        setattr(p.values, u"mp_segment_size", v * 1024 * 1024)
+        setattr(p.values, u"mp_set", True)
+    parser.add_option(u"--mp-segment-size", type=u"int", action=u"callback", metavar=_(u"number"),
+                      callback=set_mpsize)
     # volume size
     # TRANSL: Used in usage help to represent a desired number of
     # something. Example:
     # --num-retries <number>
+
+    def set_volsize(o, s, v, p):  # pylint: disable=unused-argument
+        setattr(p.values, u"volsize", v * 1024 * 1024)
+        # if mp_size was not explicity given, default it to volsize
+        if not getattr(p.values, u'mp_set', False):
+            setattr(p.values, u"mp_segment_size", int(config.mp_factor * p.values.volsize))
+
     parser.add_option(u"--volsize", type=u"int", action=u"callback", metavar=_(u"number"),
-                      callback=lambda o, s, v, p: setattr(p.values, u"volsize", v * 1024 * 1024))
+                      callback=set_volsize)
 
     # If set, collect only the file status, not the whole root.
     parser.add_option(u"--file-changed", action=u"callback", type=u"file",
                       metavar=_(u"path"), dest=u"file_changed",
                       callback=lambda o, s, v, p: setattr(p.values, u"file_changed", v.rstrip(u'/')))
 
+    # If set, skip collecting the files_changed list in statistics, nullifies --file-changed
+    parser.add_option(u"--no-files-changed", action=u"store_true", dest=u"no_files_changed")
+
+    # If set, show file changes (new, deleted, changed) in the specified backup
+    #  set (0 specifies latest, 1 specifies next latest, etc.)
+    parser.add_option(u"--show-changes-in-set", type=u"int", metavar=_(u"number"))
+
     # delay time before next try after a failure of a backend operation
     # TRANSL: Used in usage help. Example:
     # --backend-retry-delay <seconds>
     parser.add_option(u"--backend-retry-delay", type=u"int", metavar=_(u"seconds"))
 
+    # do we check remote?  requires private encryption key or correct password
+    # default is to check the remote
+    parser.add_option(u"--no-check-remote", action=u"callback", dest=u"check_remote",
+                      callback=lambda o, s, v, p: (log.Warn(
+                          _(u"Warning: Option %s is dangerous in that it allows corrupted manifests to\n"
+                            u"exist on the remote without being caught.  This could lead to problems\n"
+                            u"such as the inability to recover the backup in the future." % s),
+                          setattr(p.values, u"check_remote", False))))
+
     # parse the options
     (options, args) = parser.parse_args(arglist)
 
-    # Copy all arguments and their values to the globals module.  Don't copy
+    # Copy all arguments and their values to the config module.  Don't copy
     # attributes that are 'hidden' (start with an underscore) or whose name is
     # the empty string (used for arguments that don't directly store a value
     # by using dest="")
     for f in [x for x in dir(options) if x and not x.startswith(u"_")]:
         v = getattr(options, f)
         # Only set if v is not None because None is the default for all the
-        # variables.  If user didn't set it, we'll use defaults in globals.py
+        # variables.  If user didn't set it, we'll use defaults in config.py
         if v is not None:
-            setattr(globals, f, v)
+            setattr(config, f, v)
 
     # convert file_prefix* string
     if sys.version_info.major >= 3:
-        if isinstance(globals.file_prefix, str):
-            globals.file_prefix = bytes(globals.file_prefix, u'utf-8')
-        if isinstance(globals.file_prefix_manifest, str):
-            globals.file_prefix_manifest = bytes(globals.file_prefix_manifest, u'utf-8')
-        if isinstance(globals.file_prefix_archive, str):
-            globals.file_prefix_archive = bytes(globals.file_prefix_archive, u'utf-8')
-        if isinstance(globals.file_prefix_signature, str):
-            globals.file_prefix_signature = bytes(globals.file_prefix_signature, u'utf-8')
+        if isinstance(config.file_prefix, str):
+            config.file_prefix = bytes(config.file_prefix, u'utf-8')
+        if isinstance(config.file_prefix_manifest, str):
+            config.file_prefix_manifest = bytes(config.file_prefix_manifest, u'utf-8')
+        if isinstance(config.file_prefix_archive, str):
+            config.file_prefix_archive = bytes(config.file_prefix_archive, u'utf-8')
+        if isinstance(config.file_prefix_signature, str):
+            config.file_prefix_signature = bytes(config.file_prefix_signature, u'utf-8')
 
     # todo: this should really NOT be done here
-    socket.setdefaulttimeout(globals.timeout)
+    socket.setdefaulttimeout(config.timeout)
 
     # expect no cmd and two positional args
     cmd = u""
@@ -703,7 +794,7 @@ def parse_cmdline_options(arglist):
         full_backup = True
         num_expect = 2
     elif cmd == u"incremental":
-        globals.incremental = True
+        config.incremental = True
         num_expect = 2
     elif cmd == u"list-current-files":
         list_current = True
@@ -713,19 +804,19 @@ def parse_cmdline_options(arglist):
             arg = args.pop(0)
         except Exception:
             command_line_error(u"Missing time string for remove-older-than")
-        globals.remove_time = dup_time.genstrtotime(arg)
+        config.remove_time = dup_time.genstrtotime(arg)
         num_expect = 1
     elif cmd == u"remove-all-but-n-full" or cmd == u"remove-all-inc-of-but-n-full":
         if cmd == u"remove-all-but-n-full":
-            globals.remove_all_but_n_full_mode = True
+            config.remove_all_but_n_full_mode = True
         if cmd == u"remove-all-inc-of-but-n-full":
-            globals.remove_all_inc_of_but_n_full_mode = True
+            config.remove_all_inc_of_but_n_full_mode = True
         try:
             arg = args.pop(0)
         except Exception:
             command_line_error(u"Missing count for " + cmd)
-        globals.keep_chains = int(arg)
-        if not globals.keep_chains > 0:
+        config.keep_chains = int(arg)
+        if not config.keep_chains > 0:
             command_line_error(cmd + u" count must be > 0")
         num_expect = 1
     elif cmd == u"verify":
@@ -734,6 +825,13 @@ def parse_cmdline_options(arglist):
         replicate = True
         num_expect = 2
 
+    if cmd == u'replicate':
+        log.Warn(u'WARNING: Replicate is only minimally functional at this time\n'
+                 u'See https://gitlab.com/duplicity/duplicity/-/issues/98\n'
+                 u'for further details.  Please consider using rsync,\n'
+                 u'rclone, or other copy utilities to make a replication.\n'
+                 u'Replicate will be removed in version 2.0.\n')
+
     if len(args) != num_expect:
         command_line_error(u"Expected %d args, got %d" % (num_expect, len(args)))
 
@@ -758,19 +856,19 @@ def parse_cmdline_options(arglist):
                 command_line_error(u"Two URLs expected for replicate.")
             src_backend_url, backend_url = args[0], args[1]
         else:
-            lpath, backend_url = args_to_path_backend(args[0], args[1])  # @UnusedVariable
+            lpath, backend_url = args_to_path_backend(args[0], args[1])
     else:
         command_line_error(u"Too many arguments")
 
-    if globals.backup_name is None:
-        globals.backup_name = generate_default_backup_name(backend_url)
+    if config.backup_name is None:
+        config.backup_name = generate_default_backup_name(backend_url)
 
     # set and expand archive dir
-    set_archive_dir(expand_archive_dir(globals.archive_dir,
-                                       globals.backup_name))
+    set_archive_dir(expand_archive_dir(config.archive_dir,
+                                       config.backup_name))
 
-    log.Info(_(u"Using archive dir: %s") % (globals.archive_dir_path.uc_name,))
-    log.Info(_(u"Using backup name: %s") % (globals.backup_name,))
+    log.Info(_(u"Using archive dir: %s") % (config.archive_dir_path.uc_name,))
+    log.Info(_(u"Using backup name: %s") % (config.backup_name,))
 
     return args
 
@@ -788,7 +886,7 @@ def usage():
     be assumed to be for the benefit of translators, since they can get each string
     (paired with its preceding comment, if any) independently of the others."""
 
-    dict = {
+    trans = {
         # TRANSL: Used in usage help to represent a Unix-style path name. Example:
         # rsync://user[:password]@other_host[:port]//absolute_path
         u'absolute_path': _(u"absolute_path"),
@@ -854,6 +952,12 @@ def usage():
         # --archive-dir <path>
         u'path': _(u"path"),
 
+        # TRANSL: Used in usage help to represent a pattern for
+        # matching one or more files, as described in the documentation.
+        # Example:
+        # --exclude <pattern>
+        u'pattern': _(u"pattern"),
+
         # TRANSL: Used in usage help to represent a TCP port number. Example:
         # ftp://user[:password]@other.host[:port]/some_dir
         u'port': _(u"port"),
@@ -871,12 +975,6 @@ def usage():
         # --timeout <seconds>
         u'seconds': _(u"seconds"),
 
-        # TRANSL: Used in usage help to represent a "glob" style pattern for
-        # matching one or more files, as described in the documentation.
-        # Example:
-        # --exclude <shell_pattern>
-        u'shell_pattern': _(u"shell_pattern"),
-
         # TRANSL: Used in usage help to represent the name of a single file
         # directory or a Unix-style path to a directory. Example:
         # file:///some_dir
@@ -939,7 +1037,7 @@ def usage():
   duplicity remove-all-inc-of-but-n-full %(count)s [%(options)s] %(target_url)s
   duplicity replicate %(source_url)s %(target_url)s
 
-""" % dict
+""" % trans
 
     # TRANSL: Header in usage help
     msg = msg + _(u"Backends and their URL formats:") + u"""
@@ -952,9 +1050,14 @@ def usage():
   ftp://%(user)s[:%(password)s]@%(other_host)s[:%(port)s]/%(some_dir)s
   ftps://%(user)s[:%(password)s]@%(other_host)s[:%(port)s]/%(some_dir)s
   gdocs://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
+  for gdrive:// a <service-account-url> like the following is required
+        <serviceaccount-name>@<serviceaccount-name>.iam.gserviceaccount.com
+  gdrive://<service-account-url>/target-folder/?driveID=<SHARED DRIVE ID> (for GOOGLE Shared Drive)
+  gdrive://<service-account-url>/target-folder/?myDriveFolderID=<google-myDrive-folder-id> (for GOOGLE MyDrive)
   hsi://%(user)s[:%(password)s]@%(other_host)s[:%(port)s]/%(some_dir)s
   imap://%(user)s[:%(password)s]@%(other_host)s[:%(port)s]/%(some_dir)s
   mega://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
+  megav2://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
   mf://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
   onedrive://%(some_dir)s
   pca://%(container_name)s
@@ -972,7 +1075,7 @@ def usage():
   webdav://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
   webdavs://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
 
-""" % dict
+""" % trans
 
     # TRANSL: Header in usage help
     msg = msg + _(u"Commands:") + u"""
@@ -981,12 +1084,14 @@ def usage():
   full <%(source_dir)s> <%(target_url)s>
   incr <%(source_dir)s> <%(target_url)s>
   list-current-files <%(target_url)s>
-  restore <%(source_url)s> <%(target_dir)s>
-  remove-older-than <%(time)s> <%(target_url)s>
   remove-all-but-n-full <%(count)s> <%(target_url)s>
   remove-all-inc-of-but-n-full <%(count)s> <%(target_url)s>
+  remove-older-than <%(time)s> <%(target_url)s>
+  replicate <%(source_url)s> <%(target_url)s>
+  restore <%(source_url)s> <%(target_dir)s>
   verify <%(target_url)s> <%(source_dir)s>
-  replicate <%(source_url)s> <%(target_url)s>""" % dict
+
+""" % trans
 
     return msg
 
@@ -1003,24 +1108,24 @@ def set_archive_dir(dirstring):
         log.FatalError(_(u"Specified archive directory '%s' does not exist, "
                          u"or is not a directory") % (archive_dir_path.uc_name,),
                        log.ErrorCode.bad_archive_dir)
-    globals.archive_dir_path = archive_dir_path
+    config.archive_dir_path = archive_dir_path
 
 
 def set_sign_key(sign_key):
-    u"""Set globals.sign_key assuming proper key given"""
+    u"""Set config.sign_key assuming proper key given"""
     if not re.search(u"^(0x)?([0-9A-Fa-f]{8}|[0-9A-Fa-f]{16}|[0-9A-Fa-f]{40})$", sign_key):
         log.FatalError(_(u"Sign key should be an 8, 16 alt. 40 character hex string, like "
                          u"'AA0E73D2'.\nReceived '%s' instead.") % (sign_key,),
                        log.ErrorCode.bad_sign_key)
-    globals.gpg_profile.sign_key = sign_key
+    config.gpg_profile.sign_key = sign_key
 
 
 def set_selection():
     u"""Return selection iter starting at filename with arguments applied"""
     global select_opts, select_files
-    sel = selection.Select(globals.local_path)
+    sel = selection.Select(config.local_path)
     sel.ParseArgs(select_opts, select_files)
-    globals.select = sel.set_iter()
+    config.select = sel.set_iter()
 
 
 def args_to_path_backend(arg1, arg2):
@@ -1040,9 +1145,9 @@ page for more information.""")
         command_line_error(u"Two URLs specified.  "
                            u"One argument should be a path.")
     if arg1_is_backend:
-        return (arg2, arg1)
+        return arg2, arg1
     elif arg2_is_backend:
-        return (arg1, arg2)
+        return arg1, arg2
     else:
         raise AssertionError(u'should not be reached')
 
@@ -1056,19 +1161,19 @@ def set_backend(arg1, arg2):
     """
     path, bend = args_to_path_backend(arg1, arg2)
 
-    globals.backend = backend.get_backend(bend)
+    config.backend = backend.get_backend(bend)
 
     if path == arg2:
-        return (None, arg2)  # False?
+        return None, arg2  # False?
     else:
-        return (1, arg1)  # True?
+        return 1, arg1  # True?
 
 
 def process_local_dir(action, local_pathname):
-    u"""Check local directory, set globals.local_path"""
+    u"""Check local directory, set config.local_path"""
     local_path = path.Path(path.Path(local_pathname).get_canonical())
     if action == u"restore":
-        if (local_path.exists() and not local_path.isemptydir()) and not globals.force:
+        if (local_path.exists() and not local_path.isemptydir()) and not config.force:
             log.FatalError(_(u"Restore destination directory %s already "
                              u"exists.\nWill not overwrite.") % (local_path.uc_name,),
                            log.ErrorCode.restore_dir_exists)
@@ -1084,7 +1189,7 @@ def process_local_dir(action, local_pathname):
                            % (local_path.uc_name,),
                            log.ErrorCode.backup_dir_doesnt_exist)
 
-    globals.local_path = local_path
+    config.local_path = local_path
 
 
 def check_consistency(action):
@@ -1102,13 +1207,13 @@ def check_consistency(action):
     if action in [u"list-current", u"collection-status",
                   u"cleanup", u"remove-old", u"remove-all-but-n-full", u"remove-all-inc-of-but-n-full", u"replicate"]:
         assert_only_one([list_current, collection_status, cleanup, replicate,
-                         globals.remove_time is not None])
+                         config.remove_time is not None])
     elif action == u"restore" or action == u"verify":
         if full_backup:
-            command_line_error(u"--full option cannot be used when "
+            command_line_error(u"full option cannot be used when "
                                u"restoring or verifying")
-        elif globals.incremental:
-            command_line_error(u"--incremental option cannot be used when "
+        elif config.incremental:
+            command_line_error(u"incremental option cannot be used when "
                                u"restoring or verifying")
         if select_opts and action == u"restore":
             log.Warn(_(u"Command line warning: %s") % _(u"Selection options --exclude/--include\n"
@@ -1117,45 +1222,45 @@ def check_consistency(action):
     else:
         assert action == u"inc" or action == u"full"
         if verify:
-            command_line_error(u"--verify option cannot be used "
+            command_line_error(u"verify option cannot be used "
                                u"when backing up")
-        if globals.restore_dir:
+        if config.restore_dir:
             command_line_error(u"restore option incompatible with %s backup"
                                % (action,))
-        if sum([globals.s3_use_rrs, globals.s3_use_ia, globals.s3_use_onezone_ia]) >= 2:
+        if sum([config.s3_use_rrs, config.s3_use_ia, config.s3_use_onezone_ia]) >= 2:
             command_line_error(u"only one of --s3-use-rrs, --s3-use-ia, and --s3-use-onezone-ia may be used")
 
 
 def ProcessCommandLine(cmdline_list):
-    u"""Process command line, set globals, return action
+    u"""Process command line, set config, return action
 
     action will be "list-current", "collection-status", "cleanup",
     "remove-old", "restore", "verify", "full", or "inc".
 
     """
     # build initial gpg_profile
-    globals.gpg_profile = gpg.GPGProfile()
+    config.gpg_profile = gpg.GPGProfile()
 
     # parse command line
     args = parse_cmdline_options(cmdline_list)
 
     # if we get a different gpg-binary from the commandline then redo gpg_profile
-    if globals.gpg_binary is not None:
-        src = globals.gpg_profile
-        globals.gpg_profile = gpg.GPGProfile(
+    if config.gpg_binary is not None:
+        src = config.gpg_profile
+        config.gpg_profile = gpg.GPGProfile(
             passphrase=src.passphrase,
             sign_key=src.sign_key,
             recipients=src.recipients,
             hidden_recipients=src.hidden_recipients)
     log.Debug(_(u"GPG binary is %s, version %s") %
-              ((globals.gpg_binary or u'gpg'), globals.gpg_profile.gpg_version))
+              ((config.gpg_binary or u'gpg'), config.gpg_profile.gpg_version))
 
     # we can now try to import all the backends
     backend.import_backends()
 
     # parse_cmdline_options already verified that we got exactly 1 or 2
     # non-options arguments
-    assert len(args) >= 1 and len(args) <= 2, u"arg count should have been checked already"
+    assert 1 <= len(args) <= 2, u"arg count should have been checked already"
 
     if len(args) == 1:
         if list_current:
@@ -1164,24 +1269,24 @@ def ProcessCommandLine(cmdline_list):
             action = u"collection-status"
         elif cleanup:
             action = u"cleanup"
-        elif globals.remove_time is not None:
+        elif config.remove_time is not None:
             action = u"remove-old"
-        elif globals.remove_all_but_n_full_mode:
+        elif config.remove_all_but_n_full_mode:
             action = u"remove-all-but-n-full"
-        elif globals.remove_all_inc_of_but_n_full_mode:
+        elif config.remove_all_inc_of_but_n_full_mode:
             action = u"remove-all-inc-of-but-n-full"
         else:
             command_line_error(u"Too few arguments")
-        globals.backend = backend.get_backend(args[0])
-        if not globals.backend:
+        config.backend = backend.get_backend(args[0])
+        if not config.backend:
             log.FatalError(_(u"""Bad URL '%s'.
 Examples of URL strings are "scp://user@host.net:1234/path" and
 "file:///usr/local".  See the man page for more information.""") % (args[0],),
                            log.ErrorCode.bad_url)
     elif len(args) == 2:
         if replicate:
-            globals.src_backend = backend.get_backend(args[0])
-            globals.backend = backend.get_backend(args[1])
+            config.src_backend = backend.get_backend(args[0])
+            config.backend = backend.get_backend(args[1])
             action = u"replicate"
         else:
             # Figure out whether backup or restore
